#include "../include/except.h"
#include "../include/arena.h"
#include <string.h> // memset
#include <stdlib.h> // malloc 

const Exception out_of_memory = { "Out Of Memory" }; // OOM Exception.
                                                     
Arena arena_new(ptrdiff_t cap, int flags){
    Arena a = {0};
    a.beg   = (char*)malloc(cap);
    a.end   = a.beg ? a.beg + cap : 0;

    if(!(flags & SOFTFAIL)) RAISE(out_of_memory);
    if(!(flags & NOZERO)) memset(a.beg,0,cap);    

    return a;
}

char* arena_alloc(Arena* a, ptrdiff_t nb, ptrdiff_t align, ptrdiff_t count, int flags){
    ptrdiff_t pad = -(ptrdiff_t)a->beg & (align - 1);
    
    if (count > (a->end - a->beg - pad) / nb){
        if (flags & SOFTFAIL) return 0;
        RAISE(out_of_memory);
    }

    ptrdiff_t total  = count * nb;
    char* p      = a->beg + pad;
    a->beg      += pad + total;
    return flags & NOZERO ? p : (char*)memset(p, 0, total);
}

